package com.wily.profserv.logreader;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.lang.reflect.Constructor;
import java.util.Properties;

/**
 * @author Sylvain Gibassier
 * @copyright 2004-2005 Wily Technology, Inc.  All rights reserved.
 */

public class LineProcessorFactory {
    
    public static LineProcessor getLineProcessor(String className,PrintStream ps,String propertiesFileName) {
        LineProcessor lineProcessor = new NullLineProcessor();
        
        File propertiesFile = new File(propertiesFileName);
        if (!propertiesFile.exists()) {
            throw new LineProcessorInitializationFailedException("LogReader initialization failed: configuration file \"" + propertiesFile.getAbsolutePath() + "\" not found");
        }
        if (!propertiesFile.isFile()) {
            throw new LineProcessorInitializationFailedException("LogReader initialization failed: filesystem object \"" + propertiesFile.getAbsolutePath() + "\" is not a file");
        }
        if (!propertiesFile.canRead()) {
            throw new LineProcessorInitializationFailedException("LogReader initialization failed: cannot read configuration file \"" + propertiesFile.getAbsolutePath() + "\"");
        }
        Properties properties = new Properties();
        try {
            properties.load(new BufferedInputStream(new FileInputStream(propertiesFile)));
        } catch (IOException e) {
            throw new LineProcessorInitializationFailedException("LogReader initialization failed: cannot read configuration file \"" + propertiesFile.getAbsolutePath() + "\"",e);
        }
        
        try {
            Class lineProcessorClazz = Class.forName(className, false, Thread.currentThread().getContextClassLoader());
            if (lineProcessorClazz != null) {
                Class[] parameterTypes = { PrintStream.class,Properties.class};
                Constructor constructor = lineProcessorClazz.getConstructor(parameterTypes);
                if (constructor != null) {
                    Object[] constructorParameteres = {ps,properties};
                    lineProcessor = (LineProcessor) constructor.newInstance(constructorParameteres);
                }
            }
        } catch (ClassCastException e1) {
            throw new LineProcessorInitializationFailedException("\"" + className + "\" does not implement the interface \"LineProcessor\"",e1);
        } catch (ClassNotFoundException e1) {
            throw new LineProcessorInitializationFailedException("The class \"" + className + "\" was not found",e1);
        } catch (NoSuchMethodException e1) {
            throw new LineProcessorInitializationFailedException("The constructor with parameters (PrintStream, Properties) was not found for class \"" + className + "\"",e1);
        } catch (InstantiationException e1) {
            throw new LineProcessorInitializationFailedException("Could not create an instance of class \"" + className + "\"",e1);
        } catch (IllegalAccessException e1) {
            throw new LineProcessorInitializationFailedException("Could not access constructor with parameters (PrintStream, Properties) of class \"" + className + "\"",e1);
        } catch (Throwable e1) {
            throw new LineProcessorInitializationFailedException("Could not create an instance of the LineProcessor \"" + className + "\"",e1);
        }
        
        return lineProcessor;
    }
}
