package com.wily.profserv.logreader;

import java.lang.reflect.Method;

/**
 * @author Sylvain Gibassier
 * @copyright 2004-2005 Wily Technology, Inc.  All rights reserved.
 */

public class LogFactory {
    static private Logger logger = null;
    static private String AGENT_NAME = "[LogReader]";
    static private final Object[] EMPTY_OBJECT_ARRAY = new Object[0];
    static protected final Class[] EMPTY_CLASS_ARRAY = new Class[0];
    
    public static synchronized Logger getLogger() {
        if (logger == null) {
            // get the logger if possible
            try {
                Method agentShim_getAgent = getMethod("com.wily.introscope.agent.AgentShim","getAgent");
                Method iAgent_IAgent_getModuleFeedback = getMethod("com.wily.introscope.agent.IAgent","IAgent_getModuleFeedback");
                if (agentShim_getAgent != null && iAgent_IAgent_getModuleFeedback != null) {
                    Object agent = agentShim_getAgent.invoke(null,EMPTY_OBJECT_ARRAY);
                    if (agent != null) {
                        Object moduleFeedback = iAgent_IAgent_getModuleFeedback.invoke(agent,EMPTY_OBJECT_ARRAY);
                        if (moduleFeedback != null) {
                            logger = new AgentLogger(AGENT_NAME,moduleFeedback,new SystemOutLogger(AGENT_NAME));
                        }
                    }
                }
                
            } catch (Throwable t) {
                // no need to worry there is a default logger
                t.printStackTrace();
            }
            if (logger==null) {
                logger = new SystemOutLogger(AGENT_NAME);
            }
        }
        return logger;
    }
    
    static private Method getMethod(String className, String methodName) {
		Method method = null;
		try {
			Class clazz = Class.forName(className, false, Thread.currentThread().getContextClassLoader());
			method = clazz.getMethod(methodName, EMPTY_CLASS_ARRAY);
		} catch (Throwable t) {
			// ignore, it will return null and that's what we want
		}
		return method;
	}
}
